package com.aitrainer.api.test.diet

import com.aitrainer.api.model.diet.Diet
import com.aitrainer.api.model.diet.DietMeal
import com.aitrainer.api.repository.diet.DietRepository
import com.aitrainer.api.test.Tokenizer
import com.google.gson.Gson
import org.junit.jupiter.api.BeforeAll
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.junit.jupiter.api.extension.ExtendWith
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.MediaType
import org.springframework.test.context.junit.jupiter.SpringExtension
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.MvcResult
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get
import org.springframework.test.web.servlet.result.MockMvcResultMatchers.*
import kotlin.test.assertEquals
import kotlin.test.assertNotNull


@ExtendWith(SpringExtension::class)
@SpringBootTest
@AutoConfigureMockMvc
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class DietTest {

    @Autowired
    private lateinit var mockMvc: MockMvc

    @Autowired
    private lateinit var dietRepository: DietRepository


    private val diet = Diet(
        dietUserId = 1,
        dietText = "Test diet text",
        startDate = "2023-02-02",
        premium = 0,
    )

    private var authToken: String? = ""

    @BeforeAll
    fun initTest() {
        authToken = Tokenizer.getToken()
    }


    @Test
    fun `insert diet successfully`() {

        val mvcResult: MvcResult = mockMvc.perform(
            MockMvcRequestBuilders.post("/api/diet")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(diet))
        )
        .andExpect(status().isOk)
        .andExpect(jsonPath("$.dietUserId").value(1))
            .andReturn()

        val gson= Gson()
        val newDietJson = mvcResult.response.contentAsString
        val newDiet = gson.fromJson(newDietJson, Diet::class.java)

        mockMvc.perform(get("/api/diet/{dietUserId}", 1)
            .header("Authorization", "Bearer $authToken")
            .contentType(MediaType.APPLICATION_JSON))
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.[0].dietText").value("Test diet text"))
            .andExpect(jsonPath("$.[0].startDate").value("2023-02-02"))

        val meal = DietMeal(
            mealName = "monday|breakfast",
            meal = "Főtt tojás",
            quantity = 2.0,
            mealId = 0,
            mealDate = "2023-03-07 08:00:00"
        )
        val meal2 = DietMeal(
            mealName = "monday|lunch",
            meal = "Disznópörkölt",
            quantity = 120.0,
            mealId = 0,
            mealDate = "2023-03-07 12:00:00"
        )
        val meal3 = DietMeal(
            mealName = "monday|lunch",
            meal = "Savanyúkáposzta",
            normalizedName = "Savanyukaposzta",
            quantity = 222.0,
            quantityUnit = "gramm",
            mealId = 3,
            mealDate = "2023-03-07 12:00:00",
            serving = 1.0,
            servingUnit = "adag"

        )

        newDiet.meals.add(meal)
        newDiet.meals.add(meal2)
        newDiet.meals.add(meal3)

        //update
        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/diet/${newDiet.dietId}")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(newDiet))
        )
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.dietUserId").value(1))
            .andExpect(jsonPath("$.meals[0].meal").value("Főtt tojás"))
            .andExpect(jsonPath("$.meals[0].quantity").value(2.0))
            .andExpect(jsonPath("$.meals[0].mealDate").value("2023-03-07 08:00:00"))
            .andExpect(jsonPath("$.meals[1].mealName").value("monday|lunch"))
            .andExpect(jsonPath("$.meals[2].meal").value("Savanyúkáposzta"))
            .andExpect(jsonPath("$.meals[2].normalizedName").value("Savanyukaposzta"))
            .andExpect(jsonPath("$.meals[2].quantity").value(222.0))
            .andExpect(jsonPath("$.meals[2].mealId").value(3))
            .andExpect(jsonPath("$.meals[2].quantityUnit").value("gramm"))
            .andExpect(jsonPath("$.meals[2].servingUnit").value("adag"))

        // update no_text
        newDiet.startDate = "2023-03-20"
        newDiet.meals[0].quantity = 102.0
        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/diet/no_text/${newDiet.dietId}")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(newDiet))
        )
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.startDate").value("2023-03-20"))
            .andExpect(jsonPath("$.meals[0].quantity").value(102.0))
            .andExpect(jsonPath("$.dietText").value("Test diet text"))

         val diet2 = Diet(
            dietUserId = 2,
            dietText = "Premium Test",
            startDate = "2023-05-02",
            premium = 1,
        )

        val mvcResult2: MvcResult = mockMvc.perform(
            MockMvcRequestBuilders.post("/api/diet")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(diet2))
        )
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.dietUserId").value(2))
            .andExpect(jsonPath("$.dietText").value("Premium Test"))
            .andExpect(jsonPath("$.premium").value(1))
            .andReturn()

        val newDietJson2 = mvcResult2.response.contentAsString
        val newDiet2 = gson.fromJson(newDietJson2, Diet::class.java)

        dietRepository.delete(newDiet)
        dietRepository.delete(newDiet2)
    }

    @Test
    fun `find the last empty diet successfully`() {
        val diet: Diet? = dietRepository.findTheLastEmptyDiet(2)
        assertNotNull(diet)
        assertEquals(diet.premium, 1)
    }

    //@Test
    fun `generate premium diet successfully`() {

        val input = "Készíts egy személyre szabott heti étrendet ezekkel a paraméterekkel:\n" +
                "Férfi, 50 éves, 120kg, 170 magas, célja fogyás.\n" +
                "- Speciális étrend: ketogén\n" +
                "- Allergiák: olajos magvak\n" +
                "- Preferált ételek: magyar konyha\n" +
                "- Ezek az ételek nem szerepelhetnek az étrendben: főzelékek\n" +
                //"- napi kalóriacél: 2100 kCal\n" +
                "- Étel frekvencia: reggeli, ebéd, uzsonna, vacsora\n" +
                "- Ebéd és vacsora megegyezik. \n" +
                "- ne legyen ismétlődés az ételeknél\n" +
                "\n" +
                "A választ ebben a formátumban add meg:\n" +
                "{ \"DIET\": [\n" +
                "{\"nameDay\": enum<monday, tuesday, wednesday...>\n" +
                "\"mealTime\":enum<breakfast, lunch, snack, dinner>\n" +
                "\"meals\":[<mealNames in Hungarian>],\n" +
                //"\"quantities\":[<quantities for each mealName in gramms>],\n" +
                //"\"calories\":[<calorie values for each mealName>],\n" +
                "}" +
                "]}"

        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/diet/generate_premium/2")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(input)
        )
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.dietUserId").value(2))
            .andExpect(jsonPath("$.premium").value(1))

    }

    private fun toJson(obj: Any): String {
        return Gson().toJson(obj)
    }
}
